@@ -60,7 +60,7 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
   if (!gst_element_register (plugin, "mulawenc",
-          GST_RANK_NONE, GST_TYPE_MULAWENC) ||
+          GST_RANK_PRIMARY, GST_TYPE_MULAWENC) ||
       !gst_element_register (plugin, "mulawdec",
           GST_RANK_PRIMARY, GST_TYPE_MULAWDEC))
     return FALSE;
